# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.conf import settings  # NOQA required so settings get initialised
from appconf import AppConf
from . import utils


class AldrynSitesConf(AppConf):
    DOMAINS = {}
    SET_DOMAIN_NAME = True
    AUTO_CONFIGURE_ALLOWED_HOSTS = True

    # TODO: validate settings

    def configure(self):
        if self.configured_data['AUTO_CONFIGURE_ALLOWED_HOSTS'] and self.configured_data['DOMAINS']:
            s = self._meta.holder
            ALLOWED_HOSTS = s.ALLOWED_HOSTS
            ah_type = type(ALLOWED_HOSTS)
            ALLOWED_HOSTS = list(ALLOWED_HOSTS)
            domains = utils.get_all_domains(self.configured_data['DOMAINS'])
            for domain in domains:
                if domain not in ALLOWED_HOSTS:
                    ALLOWED_HOSTS.append(domain)
            s.ALLOWED_HOSTS = ah_type(ALLOWED_HOSTS)
        return self.configured_data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import re
from django.conf import settings
from django.contrib.sites.models import Site
import yurl


# global variable so we don't do this too often.
# I wish there were an "application ready" signal.
_has_set_site_names = False


def set_site_names(force=False):
    global _has_set_site_names
    if _has_set_site_names and not force:
        return

    _has_set_site_names = True
    sites = {site.id: site for site in Site.objects.all()}
    for site_id, site_config in settings.ALDRYN_SITES_DOMAINS.items():
        if site_id not in sites.keys():
            sites[site_id] = Site.objects.create(
                id=site_id,
                name=site_config['domain'],
                domain=site_config['domain'],
            )
        site = sites[site_id]
        if not site.name == site_config['domain']:
            if site.name == site.domain:
                # currently name and domain are the same... so change both to the new value
                site.name = site_config['domain']
            site.domain = site_config['domain']
            site.save()


def get_redirect_url(current_url, config, https=False):
    """
    priorities are (primary domain and aliases are treated the same):
        exact redirect match > exact alias match > pattern redirect match > pattern alias match
    :param current_url: the url that is being called
    :param config: redirect configuration for this url
    :param want_https: whether redircts should go to https
    :return: None for no redirect or an url to redirect to
    """
    primary_domain = config['domain']
    domains = set(config.get('aliases', [])) | set((primary_domain,))
    domain_patterns = compile_regexes(domains)
    redirect_domains = set(config.get('redirects', []))
    redirect_domain_patterns = compile_regexes(redirect_domains)
    url = yurl.URL(current_url)
    target_scheme = 'https' if https else 'http'
    redirect_url = None
    if url.is_host_ip() or url.is_host_ipv4():
        # don't redirect for ips
        return
    if url.host in domains and url.scheme == target_scheme:
        # exact host and scheme match: Nothing to do
        return
    if url.host in domains and url.scheme != target_scheme:
        # exact alias match, but scheme mismatch: redirect to change scheme
        redirect_url = url.replace(scheme=target_scheme)
    elif url.host in redirect_domains:
        # exact redirect match: redirect
        redirect_url = url.replace(scheme=target_scheme, host=primary_domain)
    elif url.host in domains:
        # exact alias match: nothing to do
        return
    elif match_any(redirect_domain_patterns, url.host):
        # pattern redirect match: redirect
        redirect_url = url.replace(scheme=target_scheme, host=primary_domain)
    elif match_any(domain_patterns, url.host):
        # pattern alias match
        if url.scheme != target_scheme:
            # pattern alias match and scheme mismatch: redirect
            redirect_url = url.replace(scheme=target_scheme)
        else:
            return

    if redirect_url:
        return '{}'.format(redirect_url)


def get_all_domains(config):
    domains = []
    for site in config.values():
        domains.append(site['domain'])
        for domain in site.get('aliases', []):
            domains.append(domain)
        for domain in site.get('redirects', []):
            domains.append(domain)
    return domains


def compile_regexes(pattern_strings):
    return [
        (re.compile(pattern_string) if not hasattr(pattern_string, 'match') else pattern_string)
        for pattern_string in pattern_strings
    ]


def match_any(patterns, string):
    for pattern in patterns:
        if pattern == string:
            # not a regex, but an exact match
            return True
        if hasattr(pattern, 'pattern') and pattern.pattern == string:
            # it's a regex, but the pattern matches 1-to-1 (no regex rules inside)
            return True
        if hasattr(pattern, 'match') and pattern.match(string):
            return True
    return False



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

__version__ = '0.5.2'
__author__ = 'Stefan Foulis'
__license__ = 'BSD'
__copyright__ = "Copyright 2014, Divio Aldryn Ltd"
__maintainer__ = 'Stefan Foulis'
__email__ = 'stefan@foulis.ch'
__url__ = 'https://github.com/aldryn/aldryn-sites/'
__status__ = 'Development'
__doc__ = 'An extension for django.contrib.sites, featuring domain redirects and automatic population of the django.contrib.sites.Site table based on settings.'  # noqa


default_app_config = 'aldryn_sites.apps.AldrynSitesConfig'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.apps import AppConfig


class AldrynSitesConfig(AppConfig):
    name = 'aldryn_sites'
    verbose_name = "Aldryn Sites"



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import re
from unittest import skipIf
from django import VERSION as DJANGO_VERSION
from django.test import TestCase

from . import utils


class AldrynSitesTestCase(TestCase):
    def test_http_redirect_url(self):
        config = {
            'domain': 'www.default.com',
            'aliases': [
                'an.other.domain.com',
            ],
            'redirects': [
                'default.com',
                'default.io',
                'www.default.io',
            ]
        }
        expected_redirects = [
            ('http://www.default.com', None),
            ('http://www.default.com/something/', None),
            ('http://default.com/', 'http://www.default.com/'),
            ('http://default.com/something/', 'http://www.default.com/something/'),
            ('http://default.io', 'http://www.default.com'),
            ('http://www.default.io/', 'http://www.default.com/'),
            ('http://an.other.domain.com/', None),

            ('https://www.default.com', 'http://www.default.com'),
            ('https://www.default.com/something/', 'http://www.default.com/something/'),
            ('https://default.com/', 'http://www.default.com/'),
            ('https://default.com/something/', 'http://www.default.com/something/'),
            ('https://default.io', 'http://www.default.com'),
            ('https://www.default.io/', 'http://www.default.com/'),
            ('https://an.other.domain.com/', 'http://an.other.domain.com/'),
        ]
        for src, dst in expected_redirects:
            self.assertEqual(dst, utils.get_redirect_url(src, config=config, https=False),
                             msg='expected {} -> {}. got {}'.format(
                                 src, dst, utils.get_redirect_url(
                                     src, config=config, https=True)))

    def test_https_redirect_url(self):
        config = {
            'domain': 'www.default.com',
            'aliases': [
                'an.other.domain.com',
            ],
            'redirects': [
                'default.com',
                'default.io',
                'www.default.io',
            ]
        }
        expected_redirects = [
            ('https://www.default.com', None),
            ('http://www.default.com', 'https://www.default.com'),

            ('https://www.default.com/something/', None),
            ('http://www.default.com/something/', 'https://www.default.com/something/'),

            ('https://default.com/', 'https://www.default.com/'),
            ('http://default.com/', 'https://www.default.com/'),

            ('https://default.com/something/', 'https://www.default.com/something/'),
            ('http://default.com/something/', 'https://www.default.com/something/'),

            ('https://default.io', 'https://www.default.com'),
            ('http://default.io', 'https://www.default.com'),

            ('https://www.default.io/', 'https://www.default.com/'),
            ('http://www.default.io/', 'https://www.default.com/'),

            ('https://an.other.domain.com/', None),
            ('http://an.other.domain.com/', 'https://an.other.domain.com/'),

        ]
        for src, dst in expected_redirects:
            self.assertEqual(dst, utils.get_redirect_url(src, config=config, https=True),
                             msg='expected {} -> {}. got {}'.format(
                                 src, dst, utils.get_redirect_url(
                                     src, config=config, https=True)))

    def test_https_pattern_priority_matches(self):
        config = {
            'domain': 'www.default.com',
            'aliases': [
                'an.other.domain.com',
                'exact.default.io',
                r'^[a-z0-9-]+\.default\.com$',
                r'^[a-z0-9-]+\.default\.me$',
            ],
            'redirects': [
                'default.com',
                'default.io',
                'exact.default.me',
                'www.default.io',
                r'^[a-z0-9-]+\.default\.com$',
                re.compile(r'^[a-z0-9-]+\.default\.io$'),  # it's possible to put a pre-compiled regex here
            ]
        }
        expected_redirects = [
            ('https://www.default.com', None),
            ('http://www.default.com', 'https://www.default.com'),

            ('https://www.default.com/something/', None),
            ('http://www.default.com/something/', 'https://www.default.com/something/'),

            ('https://default.com/', 'https://www.default.com/'),
            ('http://default.com/', 'https://www.default.com/'),

            ('https://default.com/something/', 'https://www.default.com/something/'),
            ('http://default.com/something/', 'https://www.default.com/something/'),

            ('https://default.io', 'https://www.default.com'),
            ('http://default.io', 'https://www.default.com'),

            ('https://www.default.io/', 'https://www.default.com/'),
            ('http://www.default.io/', 'https://www.default.com/'),

            ('https://an.other.domain.com/', None),
            ('http://an.other.domain.com/', 'https://an.other.domain.com/'),

            # redirect pattern should win over alias pattern
            ('https://pattern.default.com/', 'https://www.default.com/'),

            # exact alias should win over pattern redirect
            ('https://www.default.com', None),
            ('https://exact.default.io', None),

            # exact redirect should win over alias pattern
            ('https://exact.default.me', 'https://www.default.com'),
        ]
        for src, dst in expected_redirects:
            self.assertEqual(dst, utils.get_redirect_url(src, config=config, https=True),
                             msg='expected {} -> {}. got {}'.format(
                                 src, dst, utils.get_redirect_url(
                                     src, config=config, https=True)))

    @skipIf(DJANGO_VERSION >= (1, 7),
            "Does not work inside tests on this version")
    def test_auto_configure_allowed_hosts(self):
        from django.conf import settings
        for domain in ['www.example.com', 'example.com', 'an.other.domain.com']:
            self.assertTrue(domain in settings.ALLOWED_HOSTS, '{} not in ALLOWED_HOSTS'.format(domain))

    def test_auto_configure_site_domain(self):
        from django.conf import settings
        from django.contrib.sites.models import Site
        Site.objects.all().delete()
        # Django 1.7 has default site created with a signal after initial
        # migration is run. We delete it in the previous step, if it exists,
        # and want to reuse specific PK from settings.
        pk = settings.ALDRYN_SITES_DOMAINS.keys()[0]
        Site.objects.create(pk=pk, name='Acme Ltd', domain='acme.com')
        utils.set_site_names(force=True)
        s = Site.objects.get()
        self.assertTrue(s.name == 'Acme Ltd', 'site name has changed')
        self.assertTrue(s.domain == 'www.example.com', 'site domain was not set')

    def test_auto_configure_site_domain_and_name_if_same(self):
        from django.conf import settings
        from django.contrib.sites.models import Site
        Site.objects.all().delete()
        pk = settings.ALDRYN_SITES_DOMAINS.keys()[0]
        Site.objects.create(pk=pk, name='acme.com', domain='acme.com')
        utils.set_site_names(force=True)
        s = Site.objects.get()
        self.assertTrue(s.name == 'www.example.com', 'site name was not set')
        self.assertTrue(s.domain == 'www.example.com', 'site domain was not set')

    def test_auto_configure_adds_new_sites(self):
        from django.contrib.sites.models import Site
        Site.objects.all().delete()
        utils.set_site_names(force=True)
        s = Site.objects.get()
        self.assertTrue(s.name == 'www.example.com', 'site name was not set')
        self.assertTrue(s.domain == 'www.example.com', 'site domain was not set')

    def test_auto_configure_adds_multiple_new_sites_with_one_existing(self):
        from django.contrib.sites.models import Site
        with self.settings(ALDRYN_SITES_DOMAINS={
            1: {'domain': 'www.example.com'},
            2: {'domain': 'www.other.com'},
        }):
            utils.set_site_names(force=True)
            s = Site.objects.get(id=1)
            self.assertTrue(s.name == 'www.example.com', 'site name was not set')
            self.assertTrue(s.domain == 'www.example.com', 'site domain was not set')
            s = Site.objects.get(id=2)
            self.assertTrue(s.name == 'www.other.com', 'site name was not set')
            self.assertTrue(s.domain == 'www.other.com', 'site domain was not set')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from django.conf import settings
from django.http import HttpResponseRedirect
from . import utils


class SiteMiddleware(object):
    """
    redirects any alias domains to the main domain.
    Takes into account if SECURE_SSL_REDIRECT (from django-secure) is set and redirects directly to the
    https version of the main domain if that is true.

    This middleware must be BEFORE djangosecure.middleware.SecurityMiddleware in MIDDLEWARE_CLASSES, so it can
    prevent the redirect from an alias domain to it's https version by djangosecure.middleware.SecurityMiddleware,
    because you might only cover the main domain with the certificate.
    """
    def __init__(self):
        self.domains = settings.ALDRYN_SITES_DOMAINS
        self.secure_redirect = getattr(settings, 'SECURE_SSL_REDIRECT', False)
        self.site_id = getattr(settings, 'SITE_ID', 1)
        utils.set_site_names()

    def process_request(self, request):
        if self.site_id not in self.domains.keys():
            return

        current_url = '{}://{}{}'.format(
            'https' if request.is_secure() else 'http',
            request.get_host(),
            request.get_full_path(),
        )
        site_config = self.domains[self.site_id]
        redirect_url = utils.get_redirect_url(current_url,
                                              config=site_config, https=self.secure_redirect)
        if redirect_url:
            return HttpResponseRedirect(redirect_url)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from setuptools import setup, find_packages
import re

module_file = open("aldryn_sites/__init__.py").read()
metadata = dict(re.findall("__([a-z]+)__\s*=\s*'([^']+)'", module_file))

setup(
    name='aldryn-sites',
    author=metadata['author'],
    author_email=metadata['email'],
    version=metadata['version'],
    url=metadata['url'],
    license=metadata['license'],
    platforms=['OS Independent'],
    description=metadata['doc'],
    long_description=open('README.rst').read(),
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=(
        'Django>=1.5,<1.10',
        'YURL>=0.13',
        'django-appconf',
    ),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Programming Language :: Python :: 2.7',
    ],
)
